"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from apideck_unify import models, utils
from apideck_unify._hooks import HookContext
from apideck_unify.types import OptionalNullable, UNSET
from apideck_unify.utils import get_security_from_env
from datetime import date, datetime
from jsonpath import JSONPath
from typing import Any, Dict, List, Mapping, Optional, Union


class Opportunities(BaseSDK):
    def list(
        self,
        *,
        raw: Optional[bool] = False,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        cursor: OptionalNullable[str] = UNSET,
        limit: Optional[int] = 20,
        filter_: Optional[
            Union[models.OpportunitiesFilter, models.OpportunitiesFilterTypedDict]
        ] = None,
        sort: Optional[
            Union[models.OpportunitiesSort, models.OpportunitiesSortTypedDict]
        ] = None,
        pass_through: Optional[Dict[str, Any]] = None,
        fields: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.CrmOpportunitiesAllResponse]:
        r"""List opportunities

        List opportunities

        :param raw: Include raw response. Mostly used for debugging purposes
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param cursor: Cursor to start from. You can find cursors for next/previous pages in the meta.cursors property of the response.
        :param limit: Number of results to return. Minimum 1, Maximum 200, Default 20
        :param filter_: Apply filters
        :param sort: Apply sorting
        :param pass_through: Optional unmapped key/values that will be passed through to downstream as query parameters. Ie: ?pass_through[search]=leads becomes ?search=leads
        :param fields: The 'fields' parameter allows API users to specify the fields they want to include in the API response. If this parameter is not present, the API will return all available fields. If this parameter is present, only the fields specified in the comma-separated string will be included in the response. Nested properties can also be requested by using a dot notation. <br /><br />Example: `fields=name,email,addresses.city`<br /><br />In the example above, the response will only include the fields \"name\", \"email\" and \"addresses.city\". If any other fields are available, they will be excluded.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CrmOpportunitiesAllRequest(
            raw=raw,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            cursor=cursor,
            limit=limit,
            filter_=utils.get_pydantic_model(
                filter_, Optional[models.OpportunitiesFilter]
            ),
            sort=utils.get_pydantic_model(sort, Optional[models.OpportunitiesSort]),
            pass_through=pass_through,
            fields=fields,
        )

        req = self._build_request(
            method="GET",
            path="/crm/opportunities",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CrmOpportunitiesAllGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="crm.opportunitiesAll",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.CrmOpportunitiesAllResponse]:
            body = utils.unmarshal_json(http_res.text, Dict[Any, Any])
            next_cursor = JSONPath("$.meta.cursors.next").parse(body)

            if len(next_cursor) == 0:
                return None

            next_cursor = next_cursor[0]
            if next_cursor is None:
                return None

            return self.list(
                raw=raw,
                consumer_id=consumer_id,
                app_id=app_id,
                service_id=service_id,
                cursor=next_cursor,
                limit=limit,
                filter_=filter_,
                sort=sort,
                pass_through=pass_through,
                fields=fields,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.CrmOpportunitiesAllResponse(
                get_opportunities_response=utils.unmarshal_json(
                    http_res.text, Optional[models.GetOpportunitiesResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
                next=next_func,
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.BadRequestResponseData
            )
            raise models.BadRequestResponse(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnauthorizedResponseData
            )
            raise models.UnauthorizedResponse(data=response_data)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.NotFoundResponseData
            )
            raise models.NotFoundResponse(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableResponseData
            )
            raise models.UnprocessableResponse(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.CrmOpportunitiesAllResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
                next=next_func,
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def list_async(
        self,
        *,
        raw: Optional[bool] = False,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        cursor: OptionalNullable[str] = UNSET,
        limit: Optional[int] = 20,
        filter_: Optional[
            Union[models.OpportunitiesFilter, models.OpportunitiesFilterTypedDict]
        ] = None,
        sort: Optional[
            Union[models.OpportunitiesSort, models.OpportunitiesSortTypedDict]
        ] = None,
        pass_through: Optional[Dict[str, Any]] = None,
        fields: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.CrmOpportunitiesAllResponse]:
        r"""List opportunities

        List opportunities

        :param raw: Include raw response. Mostly used for debugging purposes
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param cursor: Cursor to start from. You can find cursors for next/previous pages in the meta.cursors property of the response.
        :param limit: Number of results to return. Minimum 1, Maximum 200, Default 20
        :param filter_: Apply filters
        :param sort: Apply sorting
        :param pass_through: Optional unmapped key/values that will be passed through to downstream as query parameters. Ie: ?pass_through[search]=leads becomes ?search=leads
        :param fields: The 'fields' parameter allows API users to specify the fields they want to include in the API response. If this parameter is not present, the API will return all available fields. If this parameter is present, only the fields specified in the comma-separated string will be included in the response. Nested properties can also be requested by using a dot notation. <br /><br />Example: `fields=name,email,addresses.city`<br /><br />In the example above, the response will only include the fields \"name\", \"email\" and \"addresses.city\". If any other fields are available, they will be excluded.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CrmOpportunitiesAllRequest(
            raw=raw,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            cursor=cursor,
            limit=limit,
            filter_=utils.get_pydantic_model(
                filter_, Optional[models.OpportunitiesFilter]
            ),
            sort=utils.get_pydantic_model(sort, Optional[models.OpportunitiesSort]),
            pass_through=pass_through,
            fields=fields,
        )

        req = self._build_request_async(
            method="GET",
            path="/crm/opportunities",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CrmOpportunitiesAllGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="crm.opportunitiesAll",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.CrmOpportunitiesAllResponse]:
            body = utils.unmarshal_json(http_res.text, Dict[Any, Any])
            next_cursor = JSONPath("$.meta.cursors.next").parse(body)

            if len(next_cursor) == 0:
                return None

            next_cursor = next_cursor[0]
            if next_cursor is None:
                return None

            return self.list(
                raw=raw,
                consumer_id=consumer_id,
                app_id=app_id,
                service_id=service_id,
                cursor=next_cursor,
                limit=limit,
                filter_=filter_,
                sort=sort,
                pass_through=pass_through,
                fields=fields,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.CrmOpportunitiesAllResponse(
                get_opportunities_response=utils.unmarshal_json(
                    http_res.text, Optional[models.GetOpportunitiesResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
                next=next_func,
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.BadRequestResponseData
            )
            raise models.BadRequestResponse(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnauthorizedResponseData
            )
            raise models.UnauthorizedResponse(data=response_data)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.NotFoundResponseData
            )
            raise models.NotFoundResponse(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableResponseData
            )
            raise models.UnprocessableResponse(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.CrmOpportunitiesAllResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
                next=next_func,
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def create(
        self,
        *,
        title: str,
        raw: Optional[bool] = False,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        primary_contact_id: OptionalNullable[str] = UNSET,
        description: OptionalNullable[str] = UNSET,
        type_: OptionalNullable[str] = UNSET,
        monetary_amount: OptionalNullable[float] = UNSET,
        currency: OptionalNullable[models.Currency] = UNSET,
        win_probability: OptionalNullable[float] = UNSET,
        close_date: OptionalNullable[date] = UNSET,
        loss_reason_id: OptionalNullable[str] = UNSET,
        loss_reason: OptionalNullable[str] = UNSET,
        won_reason_id: OptionalNullable[str] = UNSET,
        won_reason: OptionalNullable[str] = UNSET,
        pipeline_id: OptionalNullable[str] = UNSET,
        pipeline_stage_id: OptionalNullable[str] = UNSET,
        source_id: OptionalNullable[str] = UNSET,
        lead_id: OptionalNullable[str] = UNSET,
        lead_source: OptionalNullable[str] = UNSET,
        contact_id: OptionalNullable[str] = UNSET,
        contact_ids: Optional[List[str]] = None,
        company_id: OptionalNullable[str] = UNSET,
        company_name: OptionalNullable[str] = UNSET,
        owner_id: OptionalNullable[str] = UNSET,
        priority: OptionalNullable[str] = UNSET,
        status: OptionalNullable[str] = UNSET,
        status_id: OptionalNullable[str] = UNSET,
        tags: OptionalNullable[List[str]] = UNSET,
        custom_fields: Optional[
            Union[List[models.CustomField], List[models.CustomFieldTypedDict]]
        ] = None,
        stage_last_changed_at: OptionalNullable[datetime] = UNSET,
        pass_through: Optional[
            Union[List[models.PassThroughBody], List[models.PassThroughBodyTypedDict]]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CrmOpportunitiesAddResponse:
        r"""Create opportunity

        Create opportunity

        :param title: The title or name of the opportunity.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param primary_contact_id: The unique identifier of the primary contact associated with the opportunity.
        :param description: A description of the opportunity.
        :param type: The type of the opportunity
        :param monetary_amount: The monetary value associated with the opportunity
        :param currency: Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217).
        :param win_probability: The probability of winning the opportunity, expressed as a percentage.
        :param close_date: The actual closing date for the opportunity. If close_date is null, the opportunity is not closed yet.
        :param loss_reason_id: The unique identifier of the reason why the opportunity was lost.
        :param loss_reason: The reason why the opportunity was lost.
        :param won_reason_id: The unique identifier of the reason why the opportunity was won.
        :param won_reason: The reason why the opportunity was won.
        :param pipeline_id: The unique identifier of the pipeline associated with the opportunity
        :param pipeline_stage_id: The unique identifier of the stage in the pipeline associated with the opportunity.
        :param source_id: The unique identifier of the source of the opportunity.
        :param lead_id: The unique identifier of the lead associated with the opportunity.
        :param lead_source: The source of the lead associated with the opportunity.
        :param contact_id: The unique identifier of the contact associated with the opportunity.
        :param contact_ids: An array of unique identifiers of all contacts associated with the opportunity.
        :param company_id: The unique identifier of the company associated with the opportunity.
        :param company_name: The name of the company associated with the opportunity.
        :param owner_id: The unique identifier of the user who owns the opportunity.
        :param priority: The priority level of the opportunity.
        :param status: The current status of the opportunity.
        :param status_id: The unique identifier of the current status of the opportunity.
        :param tags:
        :param custom_fields:
        :param stage_last_changed_at: The date and time when the stage of the opportunity was last changed.
        :param pass_through: The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CrmOpportunitiesAddRequest(
            raw=raw,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            opportunity=models.OpportunityInput(
                title=title,
                primary_contact_id=primary_contact_id,
                description=description,
                type=type_,
                monetary_amount=monetary_amount,
                currency=currency,
                win_probability=win_probability,
                close_date=close_date,
                loss_reason_id=loss_reason_id,
                loss_reason=loss_reason,
                won_reason_id=won_reason_id,
                won_reason=won_reason,
                pipeline_id=pipeline_id,
                pipeline_stage_id=pipeline_stage_id,
                source_id=source_id,
                lead_id=lead_id,
                lead_source=lead_source,
                contact_id=contact_id,
                contact_ids=contact_ids,
                company_id=company_id,
                company_name=company_name,
                owner_id=owner_id,
                priority=priority,
                status=status,
                status_id=status_id,
                tags=tags,
                custom_fields=utils.get_pydantic_model(
                    custom_fields, Optional[List[models.CustomField]]
                ),
                stage_last_changed_at=stage_last_changed_at,
                pass_through=utils.get_pydantic_model(
                    pass_through, Optional[List[models.PassThroughBody]]
                ),
            ),
        )

        req = self._build_request(
            method="POST",
            path="/crm/opportunities",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CrmOpportunitiesAddGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.opportunity, False, False, "json", models.OpportunityInput
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="crm.opportunitiesAdd",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return models.CrmOpportunitiesAddResponse(
                create_opportunity_response=utils.unmarshal_json(
                    http_res.text, Optional[models.CreateOpportunityResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.BadRequestResponseData
            )
            raise models.BadRequestResponse(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnauthorizedResponseData
            )
            raise models.UnauthorizedResponse(data=response_data)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.NotFoundResponseData
            )
            raise models.NotFoundResponse(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableResponseData
            )
            raise models.UnprocessableResponse(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.CrmOpportunitiesAddResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def create_async(
        self,
        *,
        title: str,
        raw: Optional[bool] = False,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        primary_contact_id: OptionalNullable[str] = UNSET,
        description: OptionalNullable[str] = UNSET,
        type_: OptionalNullable[str] = UNSET,
        monetary_amount: OptionalNullable[float] = UNSET,
        currency: OptionalNullable[models.Currency] = UNSET,
        win_probability: OptionalNullable[float] = UNSET,
        close_date: OptionalNullable[date] = UNSET,
        loss_reason_id: OptionalNullable[str] = UNSET,
        loss_reason: OptionalNullable[str] = UNSET,
        won_reason_id: OptionalNullable[str] = UNSET,
        won_reason: OptionalNullable[str] = UNSET,
        pipeline_id: OptionalNullable[str] = UNSET,
        pipeline_stage_id: OptionalNullable[str] = UNSET,
        source_id: OptionalNullable[str] = UNSET,
        lead_id: OptionalNullable[str] = UNSET,
        lead_source: OptionalNullable[str] = UNSET,
        contact_id: OptionalNullable[str] = UNSET,
        contact_ids: Optional[List[str]] = None,
        company_id: OptionalNullable[str] = UNSET,
        company_name: OptionalNullable[str] = UNSET,
        owner_id: OptionalNullable[str] = UNSET,
        priority: OptionalNullable[str] = UNSET,
        status: OptionalNullable[str] = UNSET,
        status_id: OptionalNullable[str] = UNSET,
        tags: OptionalNullable[List[str]] = UNSET,
        custom_fields: Optional[
            Union[List[models.CustomField], List[models.CustomFieldTypedDict]]
        ] = None,
        stage_last_changed_at: OptionalNullable[datetime] = UNSET,
        pass_through: Optional[
            Union[List[models.PassThroughBody], List[models.PassThroughBodyTypedDict]]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CrmOpportunitiesAddResponse:
        r"""Create opportunity

        Create opportunity

        :param title: The title or name of the opportunity.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param primary_contact_id: The unique identifier of the primary contact associated with the opportunity.
        :param description: A description of the opportunity.
        :param type: The type of the opportunity
        :param monetary_amount: The monetary value associated with the opportunity
        :param currency: Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217).
        :param win_probability: The probability of winning the opportunity, expressed as a percentage.
        :param close_date: The actual closing date for the opportunity. If close_date is null, the opportunity is not closed yet.
        :param loss_reason_id: The unique identifier of the reason why the opportunity was lost.
        :param loss_reason: The reason why the opportunity was lost.
        :param won_reason_id: The unique identifier of the reason why the opportunity was won.
        :param won_reason: The reason why the opportunity was won.
        :param pipeline_id: The unique identifier of the pipeline associated with the opportunity
        :param pipeline_stage_id: The unique identifier of the stage in the pipeline associated with the opportunity.
        :param source_id: The unique identifier of the source of the opportunity.
        :param lead_id: The unique identifier of the lead associated with the opportunity.
        :param lead_source: The source of the lead associated with the opportunity.
        :param contact_id: The unique identifier of the contact associated with the opportunity.
        :param contact_ids: An array of unique identifiers of all contacts associated with the opportunity.
        :param company_id: The unique identifier of the company associated with the opportunity.
        :param company_name: The name of the company associated with the opportunity.
        :param owner_id: The unique identifier of the user who owns the opportunity.
        :param priority: The priority level of the opportunity.
        :param status: The current status of the opportunity.
        :param status_id: The unique identifier of the current status of the opportunity.
        :param tags:
        :param custom_fields:
        :param stage_last_changed_at: The date and time when the stage of the opportunity was last changed.
        :param pass_through: The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CrmOpportunitiesAddRequest(
            raw=raw,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            opportunity=models.OpportunityInput(
                title=title,
                primary_contact_id=primary_contact_id,
                description=description,
                type=type_,
                monetary_amount=monetary_amount,
                currency=currency,
                win_probability=win_probability,
                close_date=close_date,
                loss_reason_id=loss_reason_id,
                loss_reason=loss_reason,
                won_reason_id=won_reason_id,
                won_reason=won_reason,
                pipeline_id=pipeline_id,
                pipeline_stage_id=pipeline_stage_id,
                source_id=source_id,
                lead_id=lead_id,
                lead_source=lead_source,
                contact_id=contact_id,
                contact_ids=contact_ids,
                company_id=company_id,
                company_name=company_name,
                owner_id=owner_id,
                priority=priority,
                status=status,
                status_id=status_id,
                tags=tags,
                custom_fields=utils.get_pydantic_model(
                    custom_fields, Optional[List[models.CustomField]]
                ),
                stage_last_changed_at=stage_last_changed_at,
                pass_through=utils.get_pydantic_model(
                    pass_through, Optional[List[models.PassThroughBody]]
                ),
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/crm/opportunities",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CrmOpportunitiesAddGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.opportunity, False, False, "json", models.OpportunityInput
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="crm.opportunitiesAdd",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return models.CrmOpportunitiesAddResponse(
                create_opportunity_response=utils.unmarshal_json(
                    http_res.text, Optional[models.CreateOpportunityResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.BadRequestResponseData
            )
            raise models.BadRequestResponse(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnauthorizedResponseData
            )
            raise models.UnauthorizedResponse(data=response_data)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.NotFoundResponseData
            )
            raise models.NotFoundResponse(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableResponseData
            )
            raise models.UnprocessableResponse(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.CrmOpportunitiesAddResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def get(
        self,
        *,
        id: str,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        raw: Optional[bool] = False,
        fields: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CrmOpportunitiesOneResponse:
        r"""Get opportunity

        Get opportunity

        :param id: ID of the record you are acting upon.
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param fields: The 'fields' parameter allows API users to specify the fields they want to include in the API response. If this parameter is not present, the API will return all available fields. If this parameter is present, only the fields specified in the comma-separated string will be included in the response. Nested properties can also be requested by using a dot notation. <br /><br />Example: `fields=name,email,addresses.city`<br /><br />In the example above, the response will only include the fields \"name\", \"email\" and \"addresses.city\". If any other fields are available, they will be excluded.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CrmOpportunitiesOneRequest(
            id=id,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            raw=raw,
            fields=fields,
        )

        req = self._build_request(
            method="GET",
            path="/crm/opportunities/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CrmOpportunitiesOneGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="crm.opportunitiesOne",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.CrmOpportunitiesOneResponse(
                get_opportunity_response=utils.unmarshal_json(
                    http_res.text, Optional[models.GetOpportunityResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.BadRequestResponseData
            )
            raise models.BadRequestResponse(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnauthorizedResponseData
            )
            raise models.UnauthorizedResponse(data=response_data)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.NotFoundResponseData
            )
            raise models.NotFoundResponse(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableResponseData
            )
            raise models.UnprocessableResponse(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.CrmOpportunitiesOneResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def get_async(
        self,
        *,
        id: str,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        raw: Optional[bool] = False,
        fields: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CrmOpportunitiesOneResponse:
        r"""Get opportunity

        Get opportunity

        :param id: ID of the record you are acting upon.
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param fields: The 'fields' parameter allows API users to specify the fields they want to include in the API response. If this parameter is not present, the API will return all available fields. If this parameter is present, only the fields specified in the comma-separated string will be included in the response. Nested properties can also be requested by using a dot notation. <br /><br />Example: `fields=name,email,addresses.city`<br /><br />In the example above, the response will only include the fields \"name\", \"email\" and \"addresses.city\". If any other fields are available, they will be excluded.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CrmOpportunitiesOneRequest(
            id=id,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            raw=raw,
            fields=fields,
        )

        req = self._build_request_async(
            method="GET",
            path="/crm/opportunities/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CrmOpportunitiesOneGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="crm.opportunitiesOne",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.CrmOpportunitiesOneResponse(
                get_opportunity_response=utils.unmarshal_json(
                    http_res.text, Optional[models.GetOpportunityResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.BadRequestResponseData
            )
            raise models.BadRequestResponse(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnauthorizedResponseData
            )
            raise models.UnauthorizedResponse(data=response_data)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.NotFoundResponseData
            )
            raise models.NotFoundResponse(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableResponseData
            )
            raise models.UnprocessableResponse(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.CrmOpportunitiesOneResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def update(
        self,
        *,
        id: str,
        title: str,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        raw: Optional[bool] = False,
        primary_contact_id: OptionalNullable[str] = UNSET,
        description: OptionalNullable[str] = UNSET,
        type_: OptionalNullable[str] = UNSET,
        monetary_amount: OptionalNullable[float] = UNSET,
        currency: OptionalNullable[models.Currency] = UNSET,
        win_probability: OptionalNullable[float] = UNSET,
        close_date: OptionalNullable[date] = UNSET,
        loss_reason_id: OptionalNullable[str] = UNSET,
        loss_reason: OptionalNullable[str] = UNSET,
        won_reason_id: OptionalNullable[str] = UNSET,
        won_reason: OptionalNullable[str] = UNSET,
        pipeline_id: OptionalNullable[str] = UNSET,
        pipeline_stage_id: OptionalNullable[str] = UNSET,
        source_id: OptionalNullable[str] = UNSET,
        lead_id: OptionalNullable[str] = UNSET,
        lead_source: OptionalNullable[str] = UNSET,
        contact_id: OptionalNullable[str] = UNSET,
        contact_ids: Optional[List[str]] = None,
        company_id: OptionalNullable[str] = UNSET,
        company_name: OptionalNullable[str] = UNSET,
        owner_id: OptionalNullable[str] = UNSET,
        priority: OptionalNullable[str] = UNSET,
        status: OptionalNullable[str] = UNSET,
        status_id: OptionalNullable[str] = UNSET,
        tags: OptionalNullable[List[str]] = UNSET,
        custom_fields: Optional[
            Union[List[models.CustomField], List[models.CustomFieldTypedDict]]
        ] = None,
        stage_last_changed_at: OptionalNullable[datetime] = UNSET,
        pass_through: Optional[
            Union[List[models.PassThroughBody], List[models.PassThroughBodyTypedDict]]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CrmOpportunitiesUpdateResponse:
        r"""Update opportunity

        Update opportunity

        :param id: ID of the record you are acting upon.
        :param title: The title or name of the opportunity.
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param primary_contact_id: The unique identifier of the primary contact associated with the opportunity.
        :param description: A description of the opportunity.
        :param type: The type of the opportunity
        :param monetary_amount: The monetary value associated with the opportunity
        :param currency: Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217).
        :param win_probability: The probability of winning the opportunity, expressed as a percentage.
        :param close_date: The actual closing date for the opportunity. If close_date is null, the opportunity is not closed yet.
        :param loss_reason_id: The unique identifier of the reason why the opportunity was lost.
        :param loss_reason: The reason why the opportunity was lost.
        :param won_reason_id: The unique identifier of the reason why the opportunity was won.
        :param won_reason: The reason why the opportunity was won.
        :param pipeline_id: The unique identifier of the pipeline associated with the opportunity
        :param pipeline_stage_id: The unique identifier of the stage in the pipeline associated with the opportunity.
        :param source_id: The unique identifier of the source of the opportunity.
        :param lead_id: The unique identifier of the lead associated with the opportunity.
        :param lead_source: The source of the lead associated with the opportunity.
        :param contact_id: The unique identifier of the contact associated with the opportunity.
        :param contact_ids: An array of unique identifiers of all contacts associated with the opportunity.
        :param company_id: The unique identifier of the company associated with the opportunity.
        :param company_name: The name of the company associated with the opportunity.
        :param owner_id: The unique identifier of the user who owns the opportunity.
        :param priority: The priority level of the opportunity.
        :param status: The current status of the opportunity.
        :param status_id: The unique identifier of the current status of the opportunity.
        :param tags:
        :param custom_fields:
        :param stage_last_changed_at: The date and time when the stage of the opportunity was last changed.
        :param pass_through: The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CrmOpportunitiesUpdateRequest(
            id=id,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            raw=raw,
            opportunity=models.OpportunityInput(
                title=title,
                primary_contact_id=primary_contact_id,
                description=description,
                type=type_,
                monetary_amount=monetary_amount,
                currency=currency,
                win_probability=win_probability,
                close_date=close_date,
                loss_reason_id=loss_reason_id,
                loss_reason=loss_reason,
                won_reason_id=won_reason_id,
                won_reason=won_reason,
                pipeline_id=pipeline_id,
                pipeline_stage_id=pipeline_stage_id,
                source_id=source_id,
                lead_id=lead_id,
                lead_source=lead_source,
                contact_id=contact_id,
                contact_ids=contact_ids,
                company_id=company_id,
                company_name=company_name,
                owner_id=owner_id,
                priority=priority,
                status=status,
                status_id=status_id,
                tags=tags,
                custom_fields=utils.get_pydantic_model(
                    custom_fields, Optional[List[models.CustomField]]
                ),
                stage_last_changed_at=stage_last_changed_at,
                pass_through=utils.get_pydantic_model(
                    pass_through, Optional[List[models.PassThroughBody]]
                ),
            ),
        )

        req = self._build_request(
            method="PATCH",
            path="/crm/opportunities/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CrmOpportunitiesUpdateGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.opportunity, False, False, "json", models.OpportunityInput
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="crm.opportunitiesUpdate",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.CrmOpportunitiesUpdateResponse(
                update_opportunity_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UpdateOpportunityResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.BadRequestResponseData
            )
            raise models.BadRequestResponse(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnauthorizedResponseData
            )
            raise models.UnauthorizedResponse(data=response_data)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.NotFoundResponseData
            )
            raise models.NotFoundResponse(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableResponseData
            )
            raise models.UnprocessableResponse(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.CrmOpportunitiesUpdateResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def update_async(
        self,
        *,
        id: str,
        title: str,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        raw: Optional[bool] = False,
        primary_contact_id: OptionalNullable[str] = UNSET,
        description: OptionalNullable[str] = UNSET,
        type_: OptionalNullable[str] = UNSET,
        monetary_amount: OptionalNullable[float] = UNSET,
        currency: OptionalNullable[models.Currency] = UNSET,
        win_probability: OptionalNullable[float] = UNSET,
        close_date: OptionalNullable[date] = UNSET,
        loss_reason_id: OptionalNullable[str] = UNSET,
        loss_reason: OptionalNullable[str] = UNSET,
        won_reason_id: OptionalNullable[str] = UNSET,
        won_reason: OptionalNullable[str] = UNSET,
        pipeline_id: OptionalNullable[str] = UNSET,
        pipeline_stage_id: OptionalNullable[str] = UNSET,
        source_id: OptionalNullable[str] = UNSET,
        lead_id: OptionalNullable[str] = UNSET,
        lead_source: OptionalNullable[str] = UNSET,
        contact_id: OptionalNullable[str] = UNSET,
        contact_ids: Optional[List[str]] = None,
        company_id: OptionalNullable[str] = UNSET,
        company_name: OptionalNullable[str] = UNSET,
        owner_id: OptionalNullable[str] = UNSET,
        priority: OptionalNullable[str] = UNSET,
        status: OptionalNullable[str] = UNSET,
        status_id: OptionalNullable[str] = UNSET,
        tags: OptionalNullable[List[str]] = UNSET,
        custom_fields: Optional[
            Union[List[models.CustomField], List[models.CustomFieldTypedDict]]
        ] = None,
        stage_last_changed_at: OptionalNullable[datetime] = UNSET,
        pass_through: Optional[
            Union[List[models.PassThroughBody], List[models.PassThroughBodyTypedDict]]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CrmOpportunitiesUpdateResponse:
        r"""Update opportunity

        Update opportunity

        :param id: ID of the record you are acting upon.
        :param title: The title or name of the opportunity.
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param primary_contact_id: The unique identifier of the primary contact associated with the opportunity.
        :param description: A description of the opportunity.
        :param type: The type of the opportunity
        :param monetary_amount: The monetary value associated with the opportunity
        :param currency: Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217).
        :param win_probability: The probability of winning the opportunity, expressed as a percentage.
        :param close_date: The actual closing date for the opportunity. If close_date is null, the opportunity is not closed yet.
        :param loss_reason_id: The unique identifier of the reason why the opportunity was lost.
        :param loss_reason: The reason why the opportunity was lost.
        :param won_reason_id: The unique identifier of the reason why the opportunity was won.
        :param won_reason: The reason why the opportunity was won.
        :param pipeline_id: The unique identifier of the pipeline associated with the opportunity
        :param pipeline_stage_id: The unique identifier of the stage in the pipeline associated with the opportunity.
        :param source_id: The unique identifier of the source of the opportunity.
        :param lead_id: The unique identifier of the lead associated with the opportunity.
        :param lead_source: The source of the lead associated with the opportunity.
        :param contact_id: The unique identifier of the contact associated with the opportunity.
        :param contact_ids: An array of unique identifiers of all contacts associated with the opportunity.
        :param company_id: The unique identifier of the company associated with the opportunity.
        :param company_name: The name of the company associated with the opportunity.
        :param owner_id: The unique identifier of the user who owns the opportunity.
        :param priority: The priority level of the opportunity.
        :param status: The current status of the opportunity.
        :param status_id: The unique identifier of the current status of the opportunity.
        :param tags:
        :param custom_fields:
        :param stage_last_changed_at: The date and time when the stage of the opportunity was last changed.
        :param pass_through: The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CrmOpportunitiesUpdateRequest(
            id=id,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            raw=raw,
            opportunity=models.OpportunityInput(
                title=title,
                primary_contact_id=primary_contact_id,
                description=description,
                type=type_,
                monetary_amount=monetary_amount,
                currency=currency,
                win_probability=win_probability,
                close_date=close_date,
                loss_reason_id=loss_reason_id,
                loss_reason=loss_reason,
                won_reason_id=won_reason_id,
                won_reason=won_reason,
                pipeline_id=pipeline_id,
                pipeline_stage_id=pipeline_stage_id,
                source_id=source_id,
                lead_id=lead_id,
                lead_source=lead_source,
                contact_id=contact_id,
                contact_ids=contact_ids,
                company_id=company_id,
                company_name=company_name,
                owner_id=owner_id,
                priority=priority,
                status=status,
                status_id=status_id,
                tags=tags,
                custom_fields=utils.get_pydantic_model(
                    custom_fields, Optional[List[models.CustomField]]
                ),
                stage_last_changed_at=stage_last_changed_at,
                pass_through=utils.get_pydantic_model(
                    pass_through, Optional[List[models.PassThroughBody]]
                ),
            ),
        )

        req = self._build_request_async(
            method="PATCH",
            path="/crm/opportunities/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CrmOpportunitiesUpdateGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.opportunity, False, False, "json", models.OpportunityInput
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="crm.opportunitiesUpdate",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.CrmOpportunitiesUpdateResponse(
                update_opportunity_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UpdateOpportunityResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.BadRequestResponseData
            )
            raise models.BadRequestResponse(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnauthorizedResponseData
            )
            raise models.UnauthorizedResponse(data=response_data)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.NotFoundResponseData
            )
            raise models.NotFoundResponse(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableResponseData
            )
            raise models.UnprocessableResponse(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.CrmOpportunitiesUpdateResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def delete(
        self,
        *,
        id: str,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        raw: Optional[bool] = False,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CrmOpportunitiesDeleteResponse:
        r"""Delete opportunity

        Delete opportunity

        :param id: ID of the record you are acting upon.
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CrmOpportunitiesDeleteRequest(
            id=id,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            raw=raw,
        )

        req = self._build_request(
            method="DELETE",
            path="/crm/opportunities/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CrmOpportunitiesDeleteGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="crm.opportunitiesDelete",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.CrmOpportunitiesDeleteResponse(
                delete_opportunity_response=utils.unmarshal_json(
                    http_res.text, Optional[models.DeleteOpportunityResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.BadRequestResponseData
            )
            raise models.BadRequestResponse(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnauthorizedResponseData
            )
            raise models.UnauthorizedResponse(data=response_data)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.NotFoundResponseData
            )
            raise models.NotFoundResponse(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableResponseData
            )
            raise models.UnprocessableResponse(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.CrmOpportunitiesDeleteResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def delete_async(
        self,
        *,
        id: str,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        raw: Optional[bool] = False,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CrmOpportunitiesDeleteResponse:
        r"""Delete opportunity

        Delete opportunity

        :param id: ID of the record you are acting upon.
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CrmOpportunitiesDeleteRequest(
            id=id,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            raw=raw,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/crm/opportunities/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CrmOpportunitiesDeleteGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="crm.opportunitiesDelete",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.CrmOpportunitiesDeleteResponse(
                delete_opportunity_response=utils.unmarshal_json(
                    http_res.text, Optional[models.DeleteOpportunityResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.BadRequestResponseData
            )
            raise models.BadRequestResponse(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnauthorizedResponseData
            )
            raise models.UnauthorizedResponse(data=response_data)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.NotFoundResponseData
            )
            raise models.NotFoundResponse(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableResponseData
            )
            raise models.UnprocessableResponse(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.CrmOpportunitiesDeleteResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .currency import Currency
from .paymentunit import PaymentUnit
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from datetime import date
from enum import Enum
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class EmployeeJobStatus(str, Enum):
    r"""Indicates the status of the job."""

    ACTIVE = "active"
    INACTIVE = "inactive"
    OTHER = "other"


class EmployeeJobTypedDict(TypedDict):
    id: NotRequired[Nullable[str]]
    r"""A unique identifier for an object."""
    employee_id: NotRequired[Nullable[str]]
    r"""A unique identifier for an object."""
    title: NotRequired[Nullable[str]]
    r"""The job title of the person."""
    role: NotRequired[Nullable[str]]
    r"""The position and responsibilities of the person within the organization."""
    start_date: NotRequired[Nullable[date]]
    r"""The date on which the employee starts working in their current job role."""
    end_date: NotRequired[Nullable[date]]
    r"""The date on which the employee leaves or is expected to leave their current job role."""
    compensation_rate: NotRequired[Nullable[float]]
    r"""The rate of pay for the employee in their current job role."""
    currency: NotRequired[Nullable[Currency]]
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""
    payment_unit: NotRequired[Nullable[PaymentUnit]]
    r"""Unit of measurement for employee compensation."""
    hired_at: NotRequired[Nullable[date]]
    r"""The date on which the employee was hired by the organization"""
    is_primary: NotRequired[Nullable[bool]]
    r"""Indicates whether this the employee's primary job."""
    is_manager: NotRequired[Nullable[bool]]
    r"""Indicates whether this the employee has a manager role."""
    status: NotRequired[Nullable[EmployeeJobStatus]]
    r"""Indicates the status of the job."""
    location: NotRequired[AddressTypedDict]


class EmployeeJob(BaseModel):
    id: OptionalNullable[str] = UNSET
    r"""A unique identifier for an object."""

    employee_id: OptionalNullable[str] = UNSET
    r"""A unique identifier for an object."""

    title: OptionalNullable[str] = UNSET
    r"""The job title of the person."""

    role: OptionalNullable[str] = UNSET
    r"""The position and responsibilities of the person within the organization."""

    start_date: OptionalNullable[date] = UNSET
    r"""The date on which the employee starts working in their current job role."""

    end_date: OptionalNullable[date] = UNSET
    r"""The date on which the employee leaves or is expected to leave their current job role."""

    compensation_rate: OptionalNullable[float] = UNSET
    r"""The rate of pay for the employee in their current job role."""

    currency: OptionalNullable[Currency] = UNSET
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""

    payment_unit: OptionalNullable[PaymentUnit] = UNSET
    r"""Unit of measurement for employee compensation."""

    hired_at: OptionalNullable[date] = UNSET
    r"""The date on which the employee was hired by the organization"""

    is_primary: OptionalNullable[bool] = UNSET
    r"""Indicates whether this the employee's primary job."""

    is_manager: OptionalNullable[bool] = UNSET
    r"""Indicates whether this the employee has a manager role."""

    status: OptionalNullable[EmployeeJobStatus] = UNSET
    r"""Indicates the status of the job."""

    location: Optional[Address] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "employee_id",
            "title",
            "role",
            "start_date",
            "end_date",
            "compensation_rate",
            "currency",
            "payment_unit",
            "hired_at",
            "is_primary",
            "is_manager",
            "status",
            "location",
        ]
        nullable_fields = [
            "id",
            "employee_id",
            "title",
            "role",
            "start_date",
            "end_date",
            "compensation_rate",
            "currency",
            "payment_unit",
            "hired_at",
            "is_primary",
            "is_manager",
            "status",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class EmployeeJobInputTypedDict(TypedDict):
    title: NotRequired[Nullable[str]]
    r"""The job title of the person."""
    role: NotRequired[Nullable[str]]
    r"""The position and responsibilities of the person within the organization."""
    start_date: NotRequired[Nullable[date]]
    r"""The date on which the employee starts working in their current job role."""
    end_date: NotRequired[Nullable[date]]
    r"""The date on which the employee leaves or is expected to leave their current job role."""
    compensation_rate: NotRequired[Nullable[float]]
    r"""The rate of pay for the employee in their current job role."""
    currency: NotRequired[Nullable[Currency]]
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""
    payment_unit: NotRequired[Nullable[PaymentUnit]]
    r"""Unit of measurement for employee compensation."""
    hired_at: NotRequired[Nullable[date]]
    r"""The date on which the employee was hired by the organization"""
    is_primary: NotRequired[Nullable[bool]]
    r"""Indicates whether this the employee's primary job."""
    is_manager: NotRequired[Nullable[bool]]
    r"""Indicates whether this the employee has a manager role."""
    status: NotRequired[Nullable[EmployeeJobStatus]]
    r"""Indicates the status of the job."""
    location: NotRequired[AddressTypedDict]


class EmployeeJobInput(BaseModel):
    title: OptionalNullable[str] = UNSET
    r"""The job title of the person."""

    role: OptionalNullable[str] = UNSET
    r"""The position and responsibilities of the person within the organization."""

    start_date: OptionalNullable[date] = UNSET
    r"""The date on which the employee starts working in their current job role."""

    end_date: OptionalNullable[date] = UNSET
    r"""The date on which the employee leaves or is expected to leave their current job role."""

    compensation_rate: OptionalNullable[float] = UNSET
    r"""The rate of pay for the employee in their current job role."""

    currency: OptionalNullable[Currency] = UNSET
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""

    payment_unit: OptionalNullable[PaymentUnit] = UNSET
    r"""Unit of measurement for employee compensation."""

    hired_at: OptionalNullable[date] = UNSET
    r"""The date on which the employee was hired by the organization"""

    is_primary: OptionalNullable[bool] = UNSET
    r"""Indicates whether this the employee's primary job."""

    is_manager: OptionalNullable[bool] = UNSET
    r"""Indicates whether this the employee has a manager role."""

    status: OptionalNullable[EmployeeJobStatus] = UNSET
    r"""Indicates the status of the job."""

    location: Optional[Address] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "title",
            "role",
            "start_date",
            "end_date",
            "compensation_rate",
            "currency",
            "payment_unit",
            "hired_at",
            "is_primary",
            "is_manager",
            "status",
            "location",
        ]
        nullable_fields = [
            "title",
            "role",
            "start_date",
            "end_date",
            "compensation_rate",
            "currency",
            "payment_unit",
            "hired_at",
            "is_primary",
            "is_manager",
            "status",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .balancebyperiod import BalanceByPeriod, BalanceByPeriodTypedDict
from .currency import Currency
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class OutstandingBalanceByCurrencyTypedDict(TypedDict):
    currency: NotRequired[Nullable[Currency]]
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""
    total_amount: NotRequired[float]
    r"""Total amount of the outstanding balance."""
    balances_by_period: NotRequired[List[BalanceByPeriodTypedDict]]


class OutstandingBalanceByCurrency(BaseModel):
    currency: OptionalNullable[Currency] = UNSET
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""

    total_amount: Optional[float] = None
    r"""Total amount of the outstanding balance."""

    balances_by_period: Optional[List[BalanceByPeriod]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["currency", "total_amount", "balances_by_period"]
        nullable_fields = ["currency"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .consumermetadata import ConsumerMetadata, ConsumerMetadataTypedDict
from .links import Links, LinksTypedDict
from .meta import Meta, MetaTypedDict
from .requestcountallocation import (
    RequestCountAllocation,
    RequestCountAllocationTypedDict,
)
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
import pydantic
from pydantic import model_serializer
from typing import Any, Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class DataTypedDict(TypedDict):
    consumer_id: NotRequired[str]
    application_id: NotRequired[str]
    metadata: NotRequired[ConsumerMetadataTypedDict]
    r"""The metadata of the consumer. This is used to display the consumer in the sidebar. This is optional, but recommended."""
    aggregated_request_count: NotRequired[float]
    request_counts: NotRequired[RequestCountAllocationTypedDict]
    created: NotRequired[str]
    modified: NotRequired[str]
    request_count_updated: NotRequired[str]
    services: NotRequired[List[str]]


class Data(BaseModel):
    consumer_id: Optional[str] = None

    application_id: Optional[str] = None

    metadata: Optional[ConsumerMetadata] = None
    r"""The metadata of the consumer. This is used to display the consumer in the sidebar. This is optional, but recommended."""

    aggregated_request_count: Optional[float] = None

    request_counts: Optional[RequestCountAllocation] = None

    created: Optional[str] = None

    modified: Optional[str] = None

    request_count_updated: Optional[str] = None

    services: Optional[List[str]] = None


class GetConsumersResponseTypedDict(TypedDict):
    r"""Consumers"""

    status_code: int
    r"""HTTP Response Status Code"""
    status: str
    r"""HTTP Response Status"""
    data: List[DataTypedDict]
    meta: NotRequired[MetaTypedDict]
    r"""Response metadata"""
    links: NotRequired[LinksTypedDict]
    r"""Links to navigate to previous or next pages through the API"""
    raw: NotRequired[Nullable[Dict[str, Any]]]
    r"""Raw response from the integration when raw=true query param is provided"""


class GetConsumersResponse(BaseModel):
    r"""Consumers"""

    status_code: int
    r"""HTTP Response Status Code"""

    status: str
    r"""HTTP Response Status"""

    data: List[Data]

    meta: Optional[Meta] = None
    r"""Response metadata"""

    links: Optional[Links] = None
    r"""Links to navigate to previous or next pages through the API"""

    raw: Annotated[OptionalNullable[Dict[str, Any]], pydantic.Field(alias="_raw")] = (
        UNSET
    )
    r"""Raw response from the integration when raw=true query param is provided"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["meta", "links", "_raw"]
        nullable_fields = ["_raw"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .custommappings import CustomMappings, CustomMappingsTypedDict
from .passthroughbody import PassThroughBody, PassThroughBodyTypedDict
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from datetime import datetime
from enum import Enum
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class TimeOffRequestStatusStatus(str, Enum):
    r"""The status of the time off request."""

    REQUESTED = "requested"
    APPROVED = "approved"
    DECLINED = "declined"
    CANCELLED = "cancelled"
    DELETED = "deleted"
    OTHER = "other"


class RequestType(str, Enum):
    r"""The type of request"""

    VACATION = "vacation"
    SICK = "sick"
    PERSONAL = "personal"
    JURY_DUTY = "jury_duty"
    VOLUNTEER = "volunteer"
    BEREAVEMENT = "bereavement"
    OTHER = "other"


class Units(str, Enum):
    r"""The unit of time off requested. Possible values include: `hours`, `days`, or `other`."""

    DAYS = "days"
    HOURS = "hours"
    OTHER = "other"


class NotesModelTypedDict(TypedDict):
    employee: NotRequired[Nullable[str]]
    manager: NotRequired[Nullable[str]]


class NotesModel(BaseModel):
    employee: OptionalNullable[str] = UNSET

    manager: OptionalNullable[str] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["employee", "manager"]
        nullable_fields = ["employee", "manager"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class TimeOffRequestTypedDict(TypedDict):
    id: NotRequired[str]
    r"""A unique identifier for an object."""
    employee_id: NotRequired[Nullable[str]]
    r"""ID of the employee"""
    policy_id: NotRequired[Nullable[str]]
    r"""ID of the policy"""
    status: NotRequired[Nullable[TimeOffRequestStatusStatus]]
    r"""The status of the time off request."""
    description: NotRequired[Nullable[str]]
    r"""Description of the time off request."""
    start_date: NotRequired[Nullable[str]]
    r"""The start date of the time off request."""
    end_date: NotRequired[Nullable[str]]
    r"""The end date of the time off request."""
    request_date: NotRequired[Nullable[str]]
    r"""The date the request was made."""
    request_type: NotRequired[Nullable[RequestType]]
    r"""The type of request"""
    approval_date: NotRequired[Nullable[str]]
    r"""The date the request was approved"""
    units: NotRequired[Nullable[Units]]
    r"""The unit of time off requested. Possible values include: `hours`, `days`, or `other`."""
    amount: NotRequired[Nullable[float]]
    r"""The amount of time off requested."""
    day_part: NotRequired[Nullable[str]]
    r"""The day part of the time off request."""
    notes: NotRequired[NotesModelTypedDict]
    custom_mappings: NotRequired[Nullable[CustomMappingsTypedDict]]
    r"""When custom mappings are configured on the resource, the result is included here."""
    updated_by: NotRequired[Nullable[str]]
    r"""The user who last updated the object."""
    created_by: NotRequired[Nullable[str]]
    r"""The user who created the object."""
    updated_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was last updated."""
    created_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was created."""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""
    policy_type: NotRequired[str]
    r"""The policy type of the time off request"""


class TimeOffRequest(BaseModel):
    id: Optional[str] = None
    r"""A unique identifier for an object."""

    employee_id: OptionalNullable[str] = UNSET
    r"""ID of the employee"""

    policy_id: OptionalNullable[str] = UNSET
    r"""ID of the policy"""

    status: OptionalNullable[TimeOffRequestStatusStatus] = UNSET
    r"""The status of the time off request."""

    description: OptionalNullable[str] = UNSET
    r"""Description of the time off request."""

    start_date: OptionalNullable[str] = UNSET
    r"""The start date of the time off request."""

    end_date: OptionalNullable[str] = UNSET
    r"""The end date of the time off request."""

    request_date: OptionalNullable[str] = UNSET
    r"""The date the request was made."""

    request_type: OptionalNullable[RequestType] = UNSET
    r"""The type of request"""

    approval_date: OptionalNullable[str] = UNSET
    r"""The date the request was approved"""

    units: OptionalNullable[Units] = UNSET
    r"""The unit of time off requested. Possible values include: `hours`, `days`, or `other`."""

    amount: OptionalNullable[float] = UNSET
    r"""The amount of time off requested."""

    day_part: OptionalNullable[str] = UNSET
    r"""The day part of the time off request."""

    notes: Optional[NotesModel] = None

    custom_mappings: OptionalNullable[CustomMappings] = UNSET
    r"""When custom mappings are configured on the resource, the result is included here."""

    updated_by: OptionalNullable[str] = UNSET
    r"""The user who last updated the object."""

    created_by: OptionalNullable[str] = UNSET
    r"""The user who created the object."""

    updated_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was last updated."""

    created_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was created."""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    policy_type: Optional[str] = None
    r"""The policy type of the time off request"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "employee_id",
            "policy_id",
            "status",
            "description",
            "start_date",
            "end_date",
            "request_date",
            "request_type",
            "approval_date",
            "units",
            "amount",
            "day_part",
            "notes",
            "custom_mappings",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
            "pass_through",
            "policy_type",
        ]
        nullable_fields = [
            "employee_id",
            "policy_id",
            "status",
            "description",
            "start_date",
            "end_date",
            "request_date",
            "request_type",
            "approval_date",
            "units",
            "amount",
            "day_part",
            "custom_mappings",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class TimeOffRequestInputTypedDict(TypedDict):
    employee_id: NotRequired[Nullable[str]]
    r"""ID of the employee"""
    policy_id: NotRequired[Nullable[str]]
    r"""ID of the policy"""
    status: NotRequired[Nullable[TimeOffRequestStatusStatus]]
    r"""The status of the time off request."""
    description: NotRequired[Nullable[str]]
    r"""Description of the time off request."""
    start_date: NotRequired[Nullable[str]]
    r"""The start date of the time off request."""
    end_date: NotRequired[Nullable[str]]
    r"""The end date of the time off request."""
    request_date: NotRequired[Nullable[str]]
    r"""The date the request was made."""
    request_type: NotRequired[Nullable[RequestType]]
    r"""The type of request"""
    approval_date: NotRequired[Nullable[str]]
    r"""The date the request was approved"""
    units: NotRequired[Nullable[Units]]
    r"""The unit of time off requested. Possible values include: `hours`, `days`, or `other`."""
    amount: NotRequired[Nullable[float]]
    r"""The amount of time off requested."""
    day_part: NotRequired[Nullable[str]]
    r"""The day part of the time off request."""
    notes: NotRequired[NotesModelTypedDict]
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""
    policy_type: NotRequired[str]
    r"""The policy type of the time off request"""


class TimeOffRequestInput(BaseModel):
    employee_id: OptionalNullable[str] = UNSET
    r"""ID of the employee"""

    policy_id: OptionalNullable[str] = UNSET
    r"""ID of the policy"""

    status: OptionalNullable[TimeOffRequestStatusStatus] = UNSET
    r"""The status of the time off request."""

    description: OptionalNullable[str] = UNSET
    r"""Description of the time off request."""

    start_date: OptionalNullable[str] = UNSET
    r"""The start date of the time off request."""

    end_date: OptionalNullable[str] = UNSET
    r"""The end date of the time off request."""

    request_date: OptionalNullable[str] = UNSET
    r"""The date the request was made."""

    request_type: OptionalNullable[RequestType] = UNSET
    r"""The type of request"""

    approval_date: OptionalNullable[str] = UNSET
    r"""The date the request was approved"""

    units: OptionalNullable[Units] = UNSET
    r"""The unit of time off requested. Possible values include: `hours`, `days`, or `other`."""

    amount: OptionalNullable[float] = UNSET
    r"""The amount of time off requested."""

    day_part: OptionalNullable[str] = UNSET
    r"""The day part of the time off request."""

    notes: Optional[NotesModel] = None

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    policy_type: Optional[str] = None
    r"""The policy type of the time off request"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "employee_id",
            "policy_id",
            "status",
            "description",
            "start_date",
            "end_date",
            "request_date",
            "request_type",
            "approval_date",
            "units",
            "amount",
            "day_part",
            "notes",
            "pass_through",
            "policy_type",
        ]
        nullable_fields = [
            "employee_id",
            "policy_id",
            "status",
            "description",
            "start_date",
            "end_date",
            "request_date",
            "request_type",
            "approval_date",
            "units",
            "amount",
            "day_part",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from enum import Enum
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class AllocationType(str, Enum):
    r"""Type of entity this payment should be attributed to."""

    INVOICE = "invoice"
    ORDER = "order"
    EXPENSE = "expense"
    CREDIT_MEMO = "credit_memo"
    OVER_PAYMENT = "over_payment"
    PRE_PAYMENT = "pre_payment"
    JOURNAL_ENTRY = "journal_entry"
    OTHER = "other"
    BILL = "bill"


class AllocationTypedDict(TypedDict):
    id: NotRequired[str]
    r"""Unique identifier of entity this payment should be attributed to."""
    type: NotRequired[AllocationType]
    r"""Type of entity this payment should be attributed to."""
    code: NotRequired[str]
    amount: NotRequired[Nullable[float]]
    r"""Amount of payment that should be attributed to this allocation. If null, the total_amount will be used."""
    allocation_id: NotRequired[str]
    r"""Unique identifier of the allocation"""


class Allocation(BaseModel):
    id: Optional[str] = None
    r"""Unique identifier of entity this payment should be attributed to."""

    type: Optional[AllocationType] = None
    r"""Type of entity this payment should be attributed to."""

    code: Optional[str] = None

    amount: OptionalNullable[float] = UNSET
    r"""Amount of payment that should be attributed to this allocation. If null, the total_amount will be used."""

    allocation_id: Optional[str] = None
    r"""Unique identifier of the allocation"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["id", "type", "code", "amount", "allocation_id"]
        nullable_fields = ["amount"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class AllocationInputTypedDict(TypedDict):
    id: NotRequired[str]
    r"""Unique identifier of entity this payment should be attributed to."""
    type: NotRequired[AllocationType]
    r"""Type of entity this payment should be attributed to."""
    amount: NotRequired[Nullable[float]]
    r"""Amount of payment that should be attributed to this allocation. If null, the total_amount will be used."""
    allocation_id: NotRequired[str]
    r"""Unique identifier of the allocation"""


class AllocationInput(BaseModel):
    id: Optional[str] = None
    r"""Unique identifier of entity this payment should be attributed to."""

    type: Optional[AllocationType] = None
    r"""Type of entity this payment should be attributed to."""

    amount: OptionalNullable[float] = UNSET
    r"""Amount of payment that should be attributed to this allocation. If null, the total_amount will be used."""

    allocation_id: Optional[str] = None
    r"""Unique identifier of the allocation"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["id", "type", "amount", "allocation_id"]
        nullable_fields = ["amount"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .attachmentreferencetype import AttachmentReferenceType
from apideck_unify.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class AttachmentReferenceTypedDict(TypedDict):
    type: NotRequired[AttachmentReferenceType]
    id: NotRequired[str]
    r"""A unique identifier for an object."""


class AttachmentReference(BaseModel):
    type: Optional[AttachmentReferenceType] = None

    id: Optional[str] = None
    r"""A unique identifier for an object."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ProfitAndLossType(str, Enum):
    r"""The type of profit and loss"""

    SECTION = "Section"
    RECORD = "Record"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .passthroughbody import PassThroughBody, PassThroughBodyTypedDict
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class NoteInputTypedDict(TypedDict):
    title: NotRequired[Nullable[str]]
    r"""The title of the note"""
    content: NotRequired[Nullable[str]]
    r"""The content of the note."""
    owner_id: NotRequired[Nullable[str]]
    r"""The user that owns the note."""
    contact_id: NotRequired[Nullable[str]]
    r"""The contact that is related to the note."""
    company_id: NotRequired[Nullable[str]]
    r"""The company that is related to the note."""
    opportunity_id: NotRequired[Nullable[str]]
    r"""The opportunity that is related to the note."""
    lead_id: NotRequired[Nullable[str]]
    r"""The lead that is related to the note."""
    active: NotRequired[Nullable[bool]]
    r"""Whether the Note is active or not."""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class NoteInput(BaseModel):
    title: OptionalNullable[str] = UNSET
    r"""The title of the note"""

    content: OptionalNullable[str] = UNSET
    r"""The content of the note."""

    owner_id: OptionalNullable[str] = UNSET
    r"""The user that owns the note."""

    contact_id: OptionalNullable[str] = UNSET
    r"""The contact that is related to the note."""

    company_id: OptionalNullable[str] = UNSET
    r"""The company that is related to the note."""

    opportunity_id: OptionalNullable[str] = UNSET
    r"""The opportunity that is related to the note."""

    lead_id: OptionalNullable[str] = UNSET
    r"""The lead that is related to the note."""

    active: OptionalNullable[bool] = UNSET
    r"""Whether the Note is active or not."""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "title",
            "content",
            "owner_id",
            "contact_id",
            "company_id",
            "opportunity_id",
            "lead_id",
            "active",
            "pass_through",
        ]
        nullable_fields = [
            "title",
            "content",
            "owner_id",
            "contact_id",
            "company_id",
            "opportunity_id",
            "lead_id",
            "active",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ConnectionState(str, Enum):
    r"""[Connection state flow](#section/Connection-state)"""

    AVAILABLE = "available"
    CALLABLE = "callable"
    ADDED = "added"
    AUTHORIZED = "authorized"
    INVALID = "invalid"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from apideck_unify.utils import FieldMetadata
from datetime import datetime
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PaymentsFilterTypedDict(TypedDict):
    updated_since: NotRequired[datetime]


class PaymentsFilter(BaseModel):
    updated_since: Annotated[Optional[datetime], FieldMetadata(query=True)] = None
